@@ -2,6 +2,7 @@
  *
  * Copyright (c) 2000-2003 Intel Corporation 
  * All rights reserved. 
+ * Copyright (c) 2012 France Telecom All rights reserved. 
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met: 
@@ -232,55 +233,42 @@ int ixmlElement_setAttributeNode(
 	IXML_Node *preSib = NULL;
 	IXML_Node *nextSib = NULL;
 
-	if (element == NULL || newAttr == NULL) {
+	if (!element || !newAttr)
 		return IXML_INVALID_PARAMETER;
-	}
-
-	if (newAttr->n.ownerDocument != element->n.ownerDocument) {
+	if (newAttr->n.ownerDocument != element->n.ownerDocument)
 		return IXML_WRONG_DOCUMENT_ERR;
-	}
-
-	if (newAttr->ownerElement != NULL) {
+	if (newAttr->ownerElement)
 		return IXML_INUSE_ATTRIBUTE_ERR;
-	}
-
 	newAttr->ownerElement = element;
 	node = (IXML_Node *)newAttr;
-
 	attrNode = element->n.firstAttr;
-	while (attrNode != NULL) {
-		if (strcmp(attrNode->nodeName, node->nodeName) == 0) {
+	while (attrNode) {
+		if (!strcmp(attrNode->nodeName, node->nodeName))
 			/* Found it */
 			break;
-		} else {
+		else
 			attrNode = attrNode->nextSibling;
-		}
 	}
-
-	if (attrNode != NULL) {
+	if (attrNode) {
 		/* Already present, will replace by newAttr */
 		preSib = attrNode->prevSibling;
 		nextSib = attrNode->nextSibling;
-		if (preSib != NULL) {
+		if (preSib)
 			preSib->nextSibling = node;
-		}
-		if (nextSib != NULL) {
+		if (nextSib)
 			nextSib->prevSibling = node;
-		}
-		if (element->n.firstAttr == attrNode) {
+		if (element->n.firstAttr == attrNode)
 			element->n.firstAttr = node;
-		}
-		if (rtAttr != NULL) {
+		if (rtAttr)
 			*rtAttr = (IXML_Attr *)attrNode;
-		} else {
+		else
 			ixmlAttr_free((IXML_Attr *)attrNode);
-		}
 	} else {
 		/* Add this attribute */
-		if (element->n.firstAttr != NULL) {
+		if (element->n.firstAttr) {
 			prevAttr = element->n.firstAttr;
 			nextAttr = prevAttr->nextSibling;
-			while (nextAttr != NULL) {
+			while (nextAttr) {
 				prevAttr = nextAttr;
 				nextAttr = prevAttr->nextSibling;
 			}
@@ -292,10 +280,8 @@ int ixmlElement_setAttributeNode(
 			node->prevSibling = NULL;
 			node->nextSibling = NULL;
 		}
-
-		if (rtAttr != NULL) {
+		if (rtAttr)
 			*rtAttr = NULL;
-		}
 	}
 
 	return IXML_SUCCESS;
@@ -445,7 +431,7 @@ int ixmlElement_setAttributeNS(
 
 	/* see DOM 2 spec page 59 */
 	if ((newAttrNode.prefix != NULL && namespaceURI == NULL) ||
-	    (strcmp(newAttrNode.prefix, "xml") == 0 &&
+	    (newAttrNode.prefix != NULL && strcmp(newAttrNode.prefix, "xml") == 0 &&
 	     strcmp(namespaceURI, "http://www.w3.org/XML/1998/namespace") != 0) ||
 	    (strcmp(qualifiedName, "xmlns") == 0 &&
 	     strcmp(namespaceURI, "http://www.w3.org/2000/xmlns/") != 0)) {
@@ -492,15 +478,18 @@ int ixmlElement_setAttributeNS(
 			qualifiedName,
 			&newAttr);
 		if (rc != IXML_SUCCESS) {
+			Parser_freeNodeContent(&newAttrNode);
 			return rc;
 		}
 		newAttr->n.nodeValue = strdup(value);
 		if (newAttr->n.nodeValue == NULL) {
 			ixmlAttr_free(newAttr);
+			Parser_freeNodeContent(&newAttrNode);
 			return IXML_INSUFFICIENT_MEMORY;
 		}
-		if (ixmlElement_setAttributeNodeNS(element, newAttr, NULL) != IXML_SUCCESS) {
+		if (ixmlElement_setAttributeNodeNS(element, newAttr, &newAttr) != IXML_SUCCESS) {
 			ixmlAttr_free(newAttr);
+			Parser_freeNodeContent(&newAttrNode);
 			return IXML_FAILED;
 		}
 	}
