@@ -29,23 +29,25 @@
  *
  ******************************************************************************/
 
+/*!
+ * \addtogroup UpnpSamples
+ *
+ * @{
+ *
+ * \file
+ */
 
 #define SAMPLE_UTIL_C
+
 #include "sample_util.h"
 
-#include "tv_ctrlpt.h"
-#include "tv_device.h"
-
-#include <assert.h>
 #include <stdarg.h>
 #include <stdio.h>
 
-
 #if !UPNP_HAVE_TOOLS
 #	error "Need upnptools.h to compile samples ; try ./configure --enable-tools"
 #endif
 
-
 static int initialize_init = 1;
 static int initialize_register = 1;
 
@@ -66,12 +68,11 @@ int SampleUtil_Initialize(print_string print_function)
 		ithread_mutexattr_setkind_np(&attr, ITHREAD_MUTEX_RECURSIVE_NP);
 		ithread_mutex_init(&display_mutex, &attr);
 		ithread_mutexattr_destroy(&attr);
-
 		/* To shut up valgrind mutex warning. */
 		ithread_mutex_lock(&display_mutex);
 		gPrintFun = print_function;
 		ithread_mutex_unlock(&display_mutex);
-
+		/* Finished initializing. */
 		initialize_init = 0;
 	}
 
@@ -104,9 +105,8 @@ char *SampleUtil_GetElementValue(IXML_Element *element)
 	IXML_Node *child = ixmlNode_getFirstChild((IXML_Node *)element);
 	char *temp = NULL;
 
-	if (child != 0 && ixmlNode_getNodeType(child) == eTEXT_NODE) {
+	if (child != 0 && ixmlNode_getNodeType(child) == eTEXT_NODE)
 		temp = strdup(ixmlNode_getNodeValue(child));
-	}
 
 	return temp;
 }
@@ -118,7 +118,7 @@ IXML_NodeList *SampleUtil_GetFirstServiceList(IXML_Document *doc)
 	IXML_Node *servlistnode = NULL;
 
 	servlistnodelist =
-		ixmlDocument_getElementsByTagName( doc, "serviceList" );
+		ixmlDocument_getElementsByTagName(doc, "serviceList");
 	if (servlistnodelist && ixmlNodeList_length(servlistnodelist)) {
 		/* we only care about the first service list, from the root
 		 * device */
@@ -127,13 +127,15 @@ IXML_NodeList *SampleUtil_GetFirstServiceList(IXML_Document *doc)
 		ServiceList = ixmlElement_getElementsByTagName(
 			(IXML_Element *)servlistnode, "service");
 	}
-	if (servlistnodelist) {
+	if (servlistnodelist)
 		ixmlNodeList_free(servlistnodelist);
-	}
 
 	return ServiceList;
 }
 
+#define OLD_FIND_SERVICE_CODE
+#ifdef OLD_FIND_SERVICE_CODE
+#else
 /*
  * Obtain the service list 
  *    n == 0 the first
@@ -172,20 +174,20 @@ static IXML_NodeList *SampleUtil_GetNthServiceList(
 		 *  return (Node*) A pointer to a Node or NULL if there was an 
 		 *                  error. */
 		servlistnode = ixmlNodeList_item(servlistnodelist, n);
-
-		assert(servlistnode != 0);
-
-		/* create as list of DOM nodes */
-		ServiceList = ixmlElement_getElementsByTagName(
-			(IXML_Element *)servlistnode, "service");
+		if (!servlistnode) {
+			/* create as list of DOM nodes */
+			ServiceList = ixmlElement_getElementsByTagName(
+				(IXML_Element *)servlistnode, "service");
+		} else
+			SampleUtil_Print("%s(%d): ixmlNodeList_item(nodeList, n) returned NULL\n",
+				__FILE__, __LINE__);
 	}
-
-	if (servlistnodelist) {
+	if (servlistnodelist)
 		ixmlNodeList_free(servlistnodelist);
-	}
 
 	return ServiceList;
 }
+#endif
 
 char *SampleUtil_GetFirstDocumentItem(IXML_Document *doc, const char *item)
 {
@@ -200,34 +202,27 @@ char *SampleUtil_GetFirstDocumentItem(IXML_Document *doc, const char *item)
 		if (tmpNode) {
 			textNode = ixmlNode_getFirstChild(tmpNode);
 			if (!textNode) {
-				SampleUtil_Print("sample_util.c: (bug) "
-					"ixmlNode_getFirstChild(tmpNode) "
-					"returned NULL\n"); 
+				SampleUtil_Print("%s(%d): (BUG) ixmlNode_getFirstChild(tmpNode) returned NULL\n",
+					__FILE__, __LINE__); 
 				ret = strdup("");
 				goto epilogue;
 			}
-			if (!ixmlNode_getNodeValue(textNode)) {
-				SampleUtil_Print("ixmlNode_getNodeValue "
-					"returned NULL\n"); 
+			ret = strdup(ixmlNode_getNodeValue(textNode));
+			if (!ret) {
+				SampleUtil_Print("%s(%d): ixmlNode_getNodeValue returned NULL\n",
+					__FILE__, __LINE__); 
 				ret = strdup("");
-				goto epilogue;
-			} else {
-				ret = strdup(ixmlNode_getNodeValue(textNode));
 			}
-		} else {
-			SampleUtil_Print("ixmlNode_getFirstChild(tmpNode) "
-				"returned NULL\n");
-			goto epilogue;
-		}
-	} else {
-		SampleUtil_Print("Error finding %s in XML Node\n", item);
-		goto epilogue;
-	}
+		} else
+			SampleUtil_Print("%s(%d): ixmlNodeList_item(nodeList, 0) returned NULL\n",
+				__FILE__, __LINE__);
+	} else
+		SampleUtil_Print("%s(%d): Error finding %s in XML Node\n",
+			__FILE__, __LINE__, item);
 
 epilogue:
-	if (nodeList) {
+	if (nodeList)
 		ixmlNodeList_free(nodeList);
-	}
 
 	return ret;
 }
@@ -241,24 +236,23 @@ char *SampleUtil_GetFirstElementItem(IXML_Element *element, const char *item)
 
 	nodeList = ixmlElement_getElementsByTagName(element, (char *)item);
 	if (nodeList == NULL) {
-		SampleUtil_Print( "Error finding %s in XML Node\n", item);
-
+		SampleUtil_Print("%s(%d): Error finding %s in XML Node\n",
+			__FILE__, __LINE__, item);
 		return NULL;
 	}
 	tmpNode = ixmlNodeList_item(nodeList, 0);
-	if (tmpNode) {
-		SampleUtil_Print("Error finding %s value in XML Node\n", item);
+	if (!tmpNode) {
+		SampleUtil_Print("%s(%d): Error finding %s value in XML Node\n",
+			__FILE__, __LINE__, item);
 		ixmlNodeList_free(nodeList);
-
 		return NULL;
 	}
 	textNode = ixmlNode_getFirstChild(tmpNode);
 	ret = strdup(ixmlNode_getNodeValue(textNode));
 	if (!ret) {
-		SampleUtil_Print("Error allocating memory for %s in XML Node\n",
-			item);
+		SampleUtil_Print("%s(%d): Error allocating memory for %s in XML Node\n",
+			__FILE__, __LINE__, item);
 		ixmlNodeList_free(nodeList);
-
 		return NULL;
 	}
 	ixmlNodeList_free(nodeList);
@@ -512,7 +506,10 @@ int SampleUtil_FindAndParseService(IXML_Document *DescDoc, const char *location,
 	unsigned long length;
 	int found = 0;
 	int ret;
+#ifdef OLD_FIND_SERVICE_CODE
+#else /* OLD_FIND_SERVICE_CODE */
 	unsigned int sindex = 0;
+#endif /* OLD_FIND_SERVICE_CODE */
 	char *tempServiceType = NULL;
 	char *baseURL = NULL;
 	const char *base = NULL;
@@ -522,13 +519,13 @@ int SampleUtil_FindAndParseService(IXML_Document *DescDoc, const char *location,
 	IXML_Element *service = NULL;
 
 	baseURL = SampleUtil_GetFirstDocumentItem(DescDoc, "URLBase");
-	if (baseURL) {
+	if (baseURL)
 		base = baseURL;
-	} else {
+	else
 		base = location;
-	}
-
-	/* Top level */
+#ifdef OLD_FIND_SERVICE_CODE
+	serviceList = SampleUtil_GetFirstServiceList(DescDoc);
+#else /* OLD_FIND_SERVICE_CODE */
 	for (sindex = 0;
 	     (serviceList = SampleUtil_GetNthServiceList(DescDoc , sindex)) != NULL;
 	     sindex++) {
@@ -536,39 +533,31 @@ int SampleUtil_FindAndParseService(IXML_Document *DescDoc, const char *location,
 		relcontrolURL = NULL;
 		releventURL = NULL;
 		service = NULL;
-
-		/* serviceList = SampleUtil_GetFirstServiceList( DescDoc ); */
+#endif /* OLD_FIND_SERVICE_CODE */
 		length = ixmlNodeList_length(serviceList);
 		for (i = 0; i < length; i++) {
 			service = (IXML_Element *)ixmlNodeList_item(serviceList, i);
-			tempServiceType =
-				SampleUtil_GetFirstElementItem(
-					(IXML_Element *)service, "serviceType");
-			if (strcmp(tempServiceType, serviceType) == 0) {
+			tempServiceType = SampleUtil_GetFirstElementItem(
+				(IXML_Element *)service, "serviceType");
+			if (tempServiceType && strcmp(tempServiceType, serviceType) == 0) {
 				SampleUtil_Print("Found service: %s\n", serviceType);
-				*serviceId =
-					SampleUtil_GetFirstElementItem(service, "serviceId");
+				*serviceId = SampleUtil_GetFirstElementItem(service, "serviceId");
 				SampleUtil_Print("serviceId: %s\n", *serviceId);
-				relcontrolURL =
-					SampleUtil_GetFirstElementItem(service, "controlURL");
-				releventURL =
-					SampleUtil_GetFirstElementItem(service, "eventSubURL");
-				*controlURL =
-					malloc(strlen(base) + strlen(relcontrolURL)+1);
+				relcontrolURL = SampleUtil_GetFirstElementItem(service, "controlURL");
+				releventURL = SampleUtil_GetFirstElementItem(service, "eventSubURL");
+				*controlURL = malloc(strlen(base) + strlen(relcontrolURL) + 1);
 				if (*controlURL) {
 					ret = UpnpResolveURL(base, relcontrolURL, *controlURL);
-					if (ret != UPNP_E_SUCCESS) {
+					if (ret != UPNP_E_SUCCESS)
 						SampleUtil_Print("Error generating controlURL from %s + %s\n",
 							base, relcontrolURL);
-					}
 				}
-				*eventURL = malloc(strlen(base) + strlen(releventURL)+1);
+				*eventURL = malloc(strlen(base) + strlen(releventURL) + 1);
 				if (*eventURL) {
 					ret = UpnpResolveURL(base, releventURL, *eventURL);
-					if (ret != UPNP_E_SUCCESS) {
+					if (ret != UPNP_E_SUCCESS)
 						SampleUtil_Print("Error generating eventURL from %s + %s\n",
 							base, releventURL);
-					}
 				}
 				free(relcontrolURL);
 				free(releventURL);
@@ -582,11 +571,13 @@ int SampleUtil_FindAndParseService(IXML_Document *DescDoc, const char *location,
 		}
 		free(tempServiceType);
 		tempServiceType = NULL;
-		if (serviceList) {
+		if (serviceList)
 			ixmlNodeList_free(serviceList);
-		}
 		serviceList = NULL;
+#ifdef OLD_FIND_SERVICE_CODE
+#else /* OLD_FIND_SERVICE_CODE */
 	}
+#endif /* OLD_FIND_SERVICE_CODE */
 	free(baseURL);
 
 	return found;
@@ -605,9 +596,8 @@ int SampleUtil_Print(const char *fmt, ...)
 	va_start(ap, fmt);
 	rc = vsnprintf(buf, MAX_BUF, fmt, ap);
 	va_end(ap);
-	if (gPrintFun) {
+	if (gPrintFun)
 		gPrintFun("%s", buf);
-	}
 
 	ithread_mutex_unlock(&display_mutex);
 
@@ -618,9 +608,8 @@ void SampleUtil_StateUpdate(const char *varName, const char *varValue,
 	const char *UDN, eventType type)
 {
 	/* TBD: Add mutex here? */
-	if (gStateUpdateFun) {
+	if (gStateUpdateFun)
 		gStateUpdateFun(varName, varValue, UDN, type);
-	}
 }
 
 /*!
@@ -629,9 +618,11 @@ void SampleUtil_StateUpdate(const char *varName, const char *varValue,
 void linux_print(const char *format, ...)
 {
 	va_list argList;
+
 	va_start(argList, format);
 	vfprintf(stdout, format, argList);
 	fflush(stdout);
 	va_end(argList);
 }
 
+/*! @} UpnpSamples */
